require 'uu/os/http/http_headers'
require 'uu/os/http/http_query'

module UU
  module OS
    module HTTP

      # HTTP request.
      class HTTPRequest

        # Request URL.
        # @return [String] Request URL.
        attr_reader :url

        # HTTP method.
        # @return [Symbol] HTTP method (:post, :get, etc.)
        attr_reader :method

        # Request headers.
        # @return [UU::OS::HTTP::HTTPHeaders] Request headers.
        attr_reader :headers

        # Query parameters.
        # @return [UU::OS::HTTP::HTTPQuery] Query parameters.
        attr_reader :parameters

        # Request body. Note that in case of IO which does not support
        # rewind operation, data may not be readable since stream was
        # already consumed.
        # @return [Hash, Array, IO, String] Request body.
        attr_reader :body

        # Creates new instance of HTTPRequest.
        # @param [String] url Request URL.
        # @param [Symbol] method HTTP method.
        # @param [Hash, UU::OS::HTTP::HTTPHeaders] headers Request headers.
        # @param [Hash, UU::OS::HTTP::HTTPQuery] parameters Query parameters.
        # @param [Hash, Array, IO, String] body Request body.
        # @private
        def initialize(url, method, headers, parameters, body) # :nodoc:
          @url = url
          @method = method.to_sym.downcase
          @headers = UU::OS::HTTP::HTTPHeaders.new(headers)
          @parameters = UU::OS::HTTP::HTTPQuery.new(parameters)
          @body = body
        end

        # Override inspect to hide internal structures.
        # @private
        def inspect # :nodoc:
          "#<#{self.class.name}:0x#{(self.object_id << 1).to_s(16)}>"
        end

      end

    end
  end
end
